package com.aitrainer.api.controller

import com.aitrainer.api.model.Customer
import com.aitrainer.api.model.User
import com.aitrainer.api.service.ServiceBeans
import com.aitrainer.api.repository.CustomerRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpHeaders
import org.springframework.http.ResponseEntity
import org.springframework.security.access.annotation.Secured
import org.springframework.web.bind.annotation.*
import javax.validation.Valid


@RestController
@RequestMapping("/api")
class CustomerController ( private val customerRepository: CustomerRepository ) {

    @Autowired
    var serviceBeans: ServiceBeans? = null

    @Secured
    @GetMapping("/customers")
    fun getAllCustomers(@RequestHeader headers: HttpHeaders): List<Customer> =
            customerRepository.findAll()

    @Secured
    @PostMapping("/customers")
    fun createNewCustomer(@Valid @RequestBody customer: Customer, @RequestHeader headers: HttpHeaders): Customer =
            customerRepository.save(customer)

    @Secured
    @GetMapping("/customers/{id}")
    fun getCustomerById(@PathVariable(value = "id") customerId: Long, @RequestHeader headers: HttpHeaders): ResponseEntity<Customer> {
        return customerRepository.findById(customerId).map { customer -> 
            ResponseEntity.ok(customer)
        }.orElse(ResponseEntity.notFound().build())
    }


    @Secured
    @GetMapping("/customers/real")
    fun getRealCustomers(active: String, @RequestHeader headers: HttpHeaders): List<Customer> =
            customerRepository.findByActive(active)

    @Secured
    @PutMapping("/customers/{id}")
    fun updateCustomerById(@PathVariable(value = "id") customerId: Long,
                           @Valid @RequestBody newCustomer: Customer,
                           @RequestHeader headers: HttpHeaders): ResponseEntity<Customer> {

        return customerRepository.findById(customerId).map { existingCustomer ->
            val updatedCustomer: Customer = existingCustomer
                    .copy(name = newCustomer.name, 
                        firstname = newCustomer.firstname,
                        sex = newCustomer.sex,
                        age = newCustomer.age)
            ResponseEntity.ok().body(customerRepository.save(updatedCustomer))
        }.orElse(ResponseEntity.notFound().build())
    }


    @PostMapping("/registration")
    fun registration(@Valid @RequestBody json: String): ResponseEntity<*> {
        val customer = Customer()

        val newUser: User = User().fromJson(json)
        with (customer) {
            email = newUser.username
            password = serviceBeans!!.passwordEncoder().encode(newUser.password)
        }

        val returnCustomer: Customer? = customerRepository.findByEmail(newUser.username).let {
                if ( it == null) {
                    customerRepository.save(customer)
                } else {
                    null
                }
        }

        return if ( returnCustomer != null ) {
            ResponseEntity.ok().body(returnCustomer)
        } else {
            ResponseEntity.badRequest().body("Customer exists")
        }

    }

    @PostMapping("/login")
    fun login(@Valid @RequestBody json: String): ResponseEntity<*> {
        val customer = Customer()

        val newUser: User = User().fromJson(json)
        with (customer) {
            email = newUser.username
            password = newUser.password
        }
        val returnCustomer: Customer? = customerRepository.findByEmail(newUser.username).let {
            if ( it == null) {
                null
            } else {
                if (serviceBeans!!.passwordEncoder().matches(newUser.password, it.password)) {
                    it
                } else {
                    null
                }

            }
        }
        return if ( returnCustomer != null ) {
            ResponseEntity.ok().body(returnCustomer)
        } else {
            ResponseEntity.badRequest().body("Customer does not exist or the password is wrong")
        }
    }
}